import inspectSchema from './inspectSchema'

export const defaultMessages = {
  'type': '"{{key}}" must be a {{schema.type}}',
  'enum': '"{{key}}" must be one of: {{schema.enum}}',
  'minimum': '"{{key}}" must be less than or equal to {{schema.minimum}}',
  'exclusiveMinimum': '"{{key}}" must be less than {{schema.exclusiveMinimum}}',
  'maximum': '"{{key}}" must be larger than or equal to {{schema.maximum}}',
  'exclusiveMaximum': '"{{key}}" must be larger than or equal to {{schema.exclusiveMaximum}}',
  'multipleOf': '"{{key}}" must be multiple of {{schema.multipleOf}}',
  'minLength': '"{{key}}" length must be at least {{schema.minLength}} characters long',
  'maxLength': '"{{key}}" length must be less than or equal to {{schema.maxLength}} characters long',
  'pattern': '"{{key}}" should match pattern "{{schema.pattern}}"',
  'format': '"{{key}}" must be a valid {{schema.format}}',
  'minItems': '"{{key}}" must contain at least {{schema.minItems}} items',
  'maxItems': '"{{key}}" must contain less than or equal to {{schema.maxItems}} items',
  'additionalItems': '"{{key}}" is invalid',
  'uniqueItems': '"{{key}}" must contain unique items',
  'items': '"{{key}}" is invalid',
  'minProperties': '"{{key}}" must contain at least {{schema.minProperties}} properties',
  'maxProperties': '"{{key}}" must contain less than or equal to {{schema.maxProperties}} items',
  'required': '"{{key}}" is required',
  'properties': '"{{key}}" is invalid',
  'patternProperties': '"{{key}}" properties should match pattern "{{schema.patternProperties}}"',
  'additionalProperties': '"{{key}}" is invalid',
  'dependencies': '"{{key}}" is invalid',
  'allOf': '"{{key}}" is invalid',
  'anyOf': '"{{key}}" is invalid',
  'oneOf': '"{{key}}" is invalid',
  'not': '"{{key}}" is invalid',
  '$ref': '"{{key}}" is invalid'
}

function compileMessage (error, template, prefix = '') {
  if (typeof template === 'function') {
    return template(error)
  }
  let result = template
  let item
  for (let key in error) {
    item = error[key]
    if (key === 'enum' && Array.isArray(item)) {
      item = item.join(', ')
    }
    if (typeof item === 'object') {
      result = compileMessage(error[key], result, key)
    } else {
      result = result.replace(
        new RegExp('{{' + (prefix ? prefix + '.' + key : key) + '}}', 'g'),
        item
      )
    }
  }
  return result
}

export function buildMessage (rawError, messages, schema) {
  let propertySchema = inspectSchema(schema, rawError.path)
  if (propertySchema.type === 'array' && /\.[0-9]+$/.test(rawError.path)) {
    propertySchema = propertySchema.items
  }
  const error = Object.assign(
    rawError,
    {
      schema: propertySchema,
      key: propertySchema && propertySchema.label
        ? propertySchema.label
        : rawError.path.split('.').pop()
    }
  )
  let template = messages[error.keyword]
  if (error.message) {
    template = error.message
  }
  return compileMessage(error, template)
}

